@@ -1,6 +1,7 @@
 #! /bin/sh
 :
-# @(#) mkdepend.sh,v 1.11 1995/07/06 17:01:23 nm4 Exp
+#ident	"@(#)smail/conf/lib:RELEASE-3_2_0_96:mkdepend.sh,v 1.12 1997/05/09 20:08:37 woods Exp"
+#
 # build a list of dependencies and insert them at the end of the make file
 #
 # usage: makedepend -Idir ... makefile file.c ...
@@ -54,6 +55,8 @@ CC="`echo X_CC_X | sed -f defs.sed`"
 		#
 		#	# line-number "filename"
 		#
+		# use grep to avoid line-length limits in some seds
+		grep '^#[ 	]*[0-9]' |
 		# the compiler with the sun 3 produces extra garbage after
 		# the quoted filename.
 		sed -n 's/^#[ 	]*[0-9]*[ 	]*"\(.*\)"[ 0-9]*$/\1/p' |
