@@ -1,9 +1,10 @@
 /* pathalias -- by steve bellovin, as told to peter honeyman */
 #ifndef lint
 static char	*sccsid = "@(#)domain.c	9.5 92/08/25";
-static char	*rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_95:domain.c,v 1.2 1997/02/25 14:09:14 woods Exp";
+static char	*rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_96:domain.c,v 1.4 1997/05/26 21:43:47 woods Exp";
 #endif /* lint */
 
+#define NEED_SOCKETS			/* only for smail-3 */
 #include "def.h"
 
 /* imports */
@@ -106,10 +107,21 @@ nslookup(domain)
 		buf[n++] = '.';
 		buf[n] = 0;
 	}
-	if ((n = res_mkquery(QUERY, buf, C_IN, T_ANY, (char *) 0, 0, (struct rrec *) 0, q, sizeof(q))) < 0)
+	/*
+	 * LIES! DAMN LIES!
+	 * 
+	 * The manual page resolver(3) says arg#7 is (struct rrec *) but the
+	 * prototypes say (const char *), others say (const u_char *).
+	 * 
+	 * Note though that res_mkquery() is not used by the rest of
+	 * smail-3 if OBSOLETE_ERSOLVER is defined [XXX use res_search()].
+	 */
+	if ((n = res_mkquery(QUERY, buf, C_IN, T_ANY, (unsigned char *) NULL, 0,
+			     (struct rrec *) NULL, (unsigned char *) q,
+			     sizeof(q))) < 0)
 		die("impossible res_mkquery error");
 	errno = 0;
-	if ((n = res_send(q, n, a, sizeof(a))) < 0)
+	if ((n = res_send((unsigned char *) q, n, (unsigned char *) a, sizeof(a))) < 0)
 		die("res_send");
 	hp = (HEADER *) a;
 	if (hp->rcode == NOERROR)
