@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_95:bindlib.c,v 1.32 1997/05/07 21:43:18 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_96:bindlib.c,v 1.34 1997/05/26 21:43:58 woods Exp"
  */
 
 /*
@@ -717,6 +717,7 @@ bind_addr_work(raw_target, flags, priv, what, rt_info, error_p)
          */
 
         if (flags & BIND_DOMAIN_REQUIRED && strchr(target, '.') == NULL) {
+	    DEBUG1(DBG_ROUTE_MID, "target '%s' is not a domain.\n", target);
             return DB_NOMATCH;
         }
 
@@ -734,6 +735,7 @@ bind_addr_work(raw_target, flags, priv, what, rt_info, error_p)
             if (priv->required) {
                 if (match_full_or_end_domain(priv->required, mxdat->full_target) == NULL) {
                     mx_work_area_destructor(mxdat, FALSE);
+		    DEBUG1(DBG_ROUTE_HI, "target '%s' is not in one of the required domains.\n", target);
                     return DB_NOMATCH;
                 }
             }
@@ -972,6 +974,7 @@ static int full_mx_lookup (target, mxdat, allow_mangling, priv)
     }
 
     if (mxdat->mx_hints == NULL) {
+	DEBUG1(DBG_ROUTE_MID, "target '%s' needs null MX.\n", target);
         result = handle_null_mxs(mxdat, priv);
         if (result != DB_SUCCEED) {
             return result;
@@ -988,6 +991,14 @@ static int full_mx_lookup (target, mxdat, allow_mangling, priv)
         if (result != DB_SUCCEED) {
             return result;
         }
+	if (mxdat->mx_hints == NULL) {
+	    DEBUG1(DBG_ROUTE_MID, "target '%s' had no MXs after filtering.\n", target);
+	    mxdat->flags &= ~BIND_MX_ONLY; /* no other way to add ourselves! */
+	    result = handle_null_mxs(mxdat, priv);
+	    if (result != DB_SUCCEED) {
+        	return result;
+	    }
+	}
     }
 
     result = get_addr_hints(mxdat);
@@ -995,7 +1006,8 @@ static int full_mx_lookup (target, mxdat, allow_mangling, priv)
         return result;
     }
 
-    if (mxdat->mx_hints == NULL) {
+    if (mxdat->mx_hints == NULL && !(mxdat->flags & BIND_LOCAL_MX_OKAY)) {
+	DEBUG1(DBG_ROUTE_MID, "target '%s' has no MX hints.\n", target);
         return DB_NOMATCH;              /* Not sure if you really can get here! */
     } else {
         return DB_SUCCEED;
@@ -1052,6 +1064,8 @@ static int find_mx_records (target, mxdat, allow_mangling)
     }
     if (result == DB_SUCCEED) {
         mxdat->matched_target = COPY_STRING(cur_target);
+    } else {
+	DEBUG2(DBG_ROUTE_MID, "find_mx_records: target '%s' not matched, returning %d.\n", target, result);
     }
     return result;
 }
@@ -1136,7 +1150,7 @@ static int decode_cnames(mxdat)
                 return DB_FAIL;
             }
             dlen = dn_expand((unsigned char *)mx_it.hp, (unsigned char *)mx_it.eom, 
-                             (unsigned char *)mx_rr->rr_data, (unsigned char *)nambuf, 
+                             (unsigned char *)mx_rr->rr_data, (char *)nambuf, 
                              MAXDNAME);
             if (dlen < 0) {
                 /* format error in response packet */
@@ -1179,8 +1193,10 @@ static int handle_null_mxs(mxdat, priv)
 
     if (mxdat->flags & BIND_MX_ONLY) {
         /* We need MX records.  We have none - hence failed! */
+	DEBUG1(DBG_ROUTE_LO, "handle_null_mxs: no faking for %s with BIND_MX_ONLY!.\n", mxdat->matched_target);
         return DB_NEGMATCH;
     } else if (match_full_or_end_domain(priv->mx_domains, mxdat->matched_target) != NULL) {
+	DEBUG1(DBG_ROUTE_LO, "handle_null_mxs: mx_domians says no faking for %s!.\n", mxdat->matched_target);
         return DB_NEGMATCH;
     } else {
         /*
@@ -1197,7 +1213,6 @@ static int handle_null_mxs(mxdat, priv)
          * give it the benefit of the doubt and attempt delivery.
          */
 
-
         add_mx_hint(&(mxdat->mx_hints), 0,
                     mxdat->matched_target,
                     TRUE);
@@ -1223,7 +1238,9 @@ static int handle_null_mxs(mxdat, priv)
                     return DB_FAIL;
                 }
             }
-        }
+        } else {
+	    DEBUG2(DBG_ROUTE_LO, "handle_null_mxs: no address hints for %s [%d].\n", mxdat->matched_target, result);
+	}
 
         return result;                 /* A touch optomistic! */
 #undef mx_hint
@@ -1325,8 +1342,8 @@ static int filter_mx_hints(mxdat)
              * RRs with a preference value greater than or equal to that
              * of LOCAL's must be discarded.
              */
-            DEBUG2(DBG_ROUTE_HI, "Removed MX hint %s preference %d [Local precedence]\n", 
-                   name, precedence);
+            DEBUG3(DBG_ROUTE_HI, "Removed MX hint %s preference %d [Local precedence %d]\n", 
+                   name, precedence, mxdat->local_precedence);
             free_mx_hint(mx_a);
             continue;
         } else if (low_precedence < 0 || precedence < low_precedence) {
@@ -1339,6 +1356,7 @@ static int filter_mx_hints(mxdat)
         if (mxdat->local_precedence >= 0) {
             /* This means we rejected because the local host is the prefered MX */
             if (mxdat->flags & BIND_LOCAL_MX_OKAY) {
+		DEBUG(DBG_ROUTE_HI, "filter_mx_hints(): returning DB_SUCCEED because BIND_LOCAL_MX_OKAY.\n");
                 return DB_SUCCEED;
             } else {
                 *(mxdat->error_p) = matched_local_host(mxdat->what, mxdat->matched_target);
@@ -1432,6 +1450,7 @@ static int get_addr_hints(mxdat)
     if (mxdat->mx_hints)
         return DB_SUCCEED;
     else {
+	DEBUG1(DBG_ROUTE_HI, "get_addr_hints: %s.\n", dns_timeout ? "dns timeout" : "no hints");
         if (dns_timeout)
             return DB_AGAIN;
         else
@@ -1835,7 +1854,7 @@ getnextrr(it, additional)
     }
 
     dnamelen = dn_expand((unsigned char *)it->hp, (unsigned char *)it->eom, 
-                         (unsigned char *)dp, (unsigned char *)it->dname, MAXDNAME);
+                         (unsigned char *)dp, (char *)it->dname, MAXDNAME);
     if (dnamelen < 0) {
         return NULL;
     }
@@ -1870,7 +1889,7 @@ rewindrr(it, hp, packsize)
     /* skip over questions */
     while (qdcount > 0) {
         dnamelen = dn_expand((unsigned char *)it->hp, (unsigned char *)it->eom, 
-                             (unsigned char *)it->dp, (unsigned char *)it->dname, 
+                             (unsigned char *)it->dp, (char *)it->dname, 
                              MAXDNAME);
         if (dnamelen < 0) {
             it->ancount = it->nscount = it->arcount = 0;
@@ -1944,7 +1963,7 @@ decode_mx_rr(rr, it, name, precedence)
 
     GETSHORT(*precedence, s);
     dlen = dn_expand((unsigned char *)it->hp, (unsigned char *)it->eom, 
-                     (unsigned char *)s, (unsigned char *)nambuf, MAXDNAME);
+                     (unsigned char *)s, (char *)nambuf, MAXDNAME);
     if (dlen < 0) {
         return DB_FAIL;
     }
@@ -2188,6 +2207,8 @@ add_mx_hint(addr, precedence, name, implicit)
 {
     struct transport_hints * new_hint;
 
+    DEBUG3(DBG_ROUTE_MID, "add_mx_hint: %s MX %d hint for %s.\n",
+	   implicit ? "implicit" : "non-implicit", precedence, name)
     new_hint = new_mx_hint(precedence, COPY_STRING(name), implicit);
 #define mxhint(hint) ((struct mx_transport_hint *)((hint)->private))
     while (*addr && (! EQ("mx",(*addr)->hint_name)
