@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_95:config.c,v 1.25 1997/04/12 20:55:18 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_96:config.c,v 1.28 1997/06/01 18:26:54 woods Exp"
  */
 
 /*
@@ -97,7 +97,9 @@ int smtp_accept_queue = SMTP_ACCEPT_QUEUE; /* simultaneous SMTPs to queueonly */
 char *smtp_banner = SMTP_BANNER;	/* smtp startup banner message */
 int smtp_debug = SMTP_DEBUG;		/* allow DEBUG command in SMTP */
 int smtp_info = SMTP_INFO;		/* allow EXPN/VRFY commanda in SMTP */
-char *smtp_remote_allow = NULL;		/* ip list that can send remote smtp */
+char *smtp_remote_allow = SMTP_REMOTE_ALLOW; /* ip list that can send remote smtp */
+int smtp_hello_verify = SMTP_HELLO_VERIFY; /* verify HELO/EHLO host has A RR matching sockaddr */
+int smtp_sender_verify_mx_only = SMTP_SENDER_VERIFY_MX_ONLY; /* require MX RRs for MAIL FROM: hosts */
 char *sender_env_variable = SENDER_ENV_VARIABLE; /* env variable naming user */
 int switch_percent_and_bang = SWITCH_PERCENT_AND_BANG;
 					/* switch precedence of % and ! */
@@ -113,3 +115,29 @@ long smtp_receive_message_timeout = SMTP_RECEIVE_MESSAGE_TIMEOUT;
 char *auth_domains = AUTH_DOMAINS;	/* authoritative domain list */
 long rfc1413_query_timeout = -1;	/* Timeout on RFC1413 queries - initially disabled */
 long resolve_timeout = RESOLVE_TIMEOUT;	/* timeout on directors/routers */
+
+#ifdef HAVE_LIBWRAP
+
+# include <syslog.h>			/* XXX conflicts with stuff in "logs.h" */
+
+# ifndef LIBWRAP_ALLOW_FACILITY
+#  define LIBWRAP_ALLOW_FACILITY	LOG_AUTH
+# endif
+# ifndef LIBWRAP_ALLOW_SEVERITY
+#  define LIBWRAP_ALLOW_SEVERITY	LOG_INFO
+# endif
+# ifndef LIBWRAP_DENY_FACILITY
+#  define LIBWRAP_DENY_FACILITY		LOG_AUTH
+# endif
+# ifndef LIBWRAP_DENY_SEVERITY
+#  define LIBWRAP_DENY_SEVERITY		LOG_WARNING
+# endif
+
+/*
+ * These badly named global variables are used by TCP Wrappers libwrap.a and
+ * cannot yet be changed in the smail config file.
+ */
+int allow_severity = LIBWRAP_ALLOW_FACILITY | LIBWRAP_ALLOW_SEVERITY;
+int deny_severity = LIBWRAP_DENY_FACILITY | LIBWRAP_DENY_SEVERITY;
+
+#endif /* HAVE_LIBWRAP */
