@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_95:config.h,v 1.46 1997/05/05 15:32:00 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_96:config.h,v 1.50 1997/06/03 02:01:44 woods Exp"
  */
 
 /*
@@ -631,21 +631,40 @@
 # define RECEIVED_FIELD	\
 "Received: \
 ${if def:sender_host\
-	{from $sender_host${if def:sender_host_addr\
-		{(really [$sender_host_addr])}} }\
+ {from $sender_host${if def:sender_host_addr\
+	{(src addr [$sender_host_addr])}} }\
  else {${if def:sender_host_addr\
-	{from [$sender_host_addr] }}}}\
+	{from [$sender_host_addr] }\
+ else {${if origin:local\
+	{from localhost }}}}}}\
+${if def:message_size\
+	{($message_size bytes) }}\
 by $primary_name\n\t\
 ${if def:sender_program\
 	{via $sender_program }}\
 ${if def:sender_proto\
-	{with $sender_proto}\
- else {with stdio}}\
+	{with P\\:$sender_proto}\
+ else {with P:stdio}}\
+${if def:director\
+	{/D:$director}}\
+${if def:router\
+	{/R:$router}}\
+${if def:transport\
+	{/T:$transport}}\n\t\
+${if or{{def:sender} {def:owner}}\
+ {(${if def:sender\
+	{sender: <$sender>}\
+}${if and{{def:sender} {def:owner}}\
+	{ }\
+}${if def:owner\
+	{owner: <$owner>}}) }}\
 ${if def:ident_sender\
-	{ (ident $ident_sender using $ident_method)}}\n\t\
+	{(ident <$ident_sender> using $ident_method)}\
+ else {${if origin:local\
+	{(ident <$sender> using unix)}}}}\n\t\
 id <$message_id@$primary_name>\n\t\
 ${if def:input_addr\
-	{for ${top:input_addr}}\
+	{for ${if origin:local {<}}${top:input_addr}${if origin:local {>}}}\
  else {for <unknown>}}\
 ; $spool_date\n\t\
 ($version_string built $compile_date)"
@@ -688,7 +707,10 @@ ${if def:input_addr\
  * NOTE: This can be set with the config file value "return_path_field"
  */
 #ifndef	RETURN_PATH_FIELD
-# define RETURN_PATH_FIELD "Return-Path: <$sender>"
+# define RETURN_PATH_FIELD "Return-Path: <\
+${if and{{origin:local} {def:owner}}\
+	{$owner}\
+ else {$sender}}>"
 #endif
 
 /*
@@ -744,6 +766,36 @@ ${if def:input_addr\
 # define SMTP_RECEIVE_MESSAGE_TIMEOUT	(2*60*60)	/* 2 hours */
 #endif
 
+/*
+ * The default list of networks from which to allow remote relay.
+ *
+ * NOTE: the keyword "localnet" is replaced by the result of
+ * inet_netof(local_sa.sin_addr) [where local_sa is filled by
+ * getsockname(fileno(in)) for each connection].  This gives effectively
+ * a wildcard pattern that matches a classic IP network for the local
+ * address of the incoming connection.
+ */
+
+#ifndef SMTP_REMOTE_ALLOW
+# define SMTP_REMOTE_ALLOW		"localnet" /* special */
+#endif
+
+/*
+ * Whether or not we verify HELO/EHLO host has A RR matching sockaddr
+ */
+
+#ifndef SMTP_HELLO_VERIFY
+# define SMTP_HELLO_VERIFY		1 	/* 1123 says no, but... */
+#endif
+
+/*
+ * Whether or not MX RRs are required for MAIL FROM: hosts
+ */
+
+#ifndef SMTP_SENDER_VERIFY_MX_ONLY
+# define SMTP_SENDER_VERIFY_MX_ONLY	0 	/* XXX give the benefit of doubt */
+#endif
+
 /*
  * Timeout of directors and routers for multiple defers
  *
@@ -1101,7 +1153,7 @@ ${if def:input_addr\
  */
 #if defined(USE_LOCAL_MAIL_COMMAND) && !defined(NO_USE_LOCAL_MAIL_COMMAND)
 # if !defined(LOCAL_MAIL_COMMAND)
-#  ifdef UNIX_BSD4_4
+#  if defined(UNIX_BSD4_4) || defined(BSDI)
 #   define LOCAL_MAIL_COMMAND "/usr/libexec/mail.local -f ${shquote:sender} $(${lc:user}$)"
 #  else
 #   define LOCAL_MAIL_COMMAND "/bin/mail -d $(${lc:user}$)"
