@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_95:expand.c,v 1.22 1997/04/12 01:47:03 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_96:expand.c,v 1.23 1997/05/27 17:57:29 woods Exp"
  */
 
 /*
@@ -1076,6 +1076,9 @@ substitute(addr, file, found, var, len)
     if (MATCH("sender") || MATCH("from")) {
 	return sender;
     }
+    if (MATCH("owner")) {
+	return addr ? (addr->owner) : NULL;
+    }
     if (MATCH("sender_name") || MATCH("fullname")) {
 	if (sender_name == NULL && islocal) {
 	    getfullname();
