@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_95:extern.h,v 1.51 1997/04/21 17:55:32 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_96:extern.h,v 1.53 1997/06/01 18:27:42 woods Exp"
  */
 
 /*
@@ -95,6 +95,9 @@ extern int smtp_accept_queue;		/* simultaneous SMTPs until queueonly */
 extern int smtp_debug;			/* allow DEBUG command in SMTP */
 extern int smtp_info;			/* allow EXPN/VRFY commands in SMTP */
 extern char *smtp_remote_allow;		/* ip list that can send remote smtp */
+extern char *smtp_local_net;		/* the local network calculated from src. addr */
+extern int smtp_hello_verify;		/* verify HELO/EHLO host has A RR matching sockaddr */
+extern int smtp_sender_verify_mx_only;	/* require MX RRs for MAIL FROM: hosts */
 extern char *sender_env_variable;	/* env variable naming user */
 extern int switch_percent_and_bang;	/* switch precedence of % and ! */
 extern int error_copy_postmaster;	/* copy postmaster on errors */
