@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_95:header.c,v 1.24 1997/03/09 05:46:55 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_96:header.c,v 1.26 1997/05/14 17:52:09 woods Exp"
  */
 
 /*
@@ -45,6 +45,7 @@ static char *from_fn;			/* name of From: field */
 static char *sender_fn;			/* name of the Sender: field */
 static char *cc_fn;			/* name of the Cc: field */
 static char *bcc_fn;			/* name of the Bcc: field */
+static char *date_fn;			/* name of the Date: field */
 static char *message_id_fn;		/* name of the Message-Id: field */
 static int found_to;			/* TRUE if To: field was found */
 static int found_message_id;		/* TRUE if Message-Id: field found */
@@ -141,7 +142,7 @@ process_header(rq)
 			"Apparently-From";
 	}
 	new_field(build_from_field(from_fn));
-    } else if (islocal && found_from) {
+    } else if (islocal && found_from && !found_sender) {
 	/*
 	 * With no or multiple addresses in the From: field, always
 	 * generate a Sender: field.
@@ -262,6 +263,7 @@ check_for_resent()
 	sender_fn = "Resent-Sender";
 	cc_fn = "Resent-Cc";
 	bcc_fn = "Resent-Bcc";
+	date_fn = "Resent-Date";
 	message_id_fn = "Resent-Message-Id";
     } else {
 	to_fn = "To";
@@ -269,6 +271,7 @@ check_for_resent()
 	sender_fn = "Sender";
 	cc_fn = "Cc";
 	bcc_fn = "Bcc";
+	date_fn = "Date";
 	message_id_fn = "Message-Id";
     }
 }
@@ -326,7 +329,7 @@ scan_header(rq, error)
 				     (char *)NULL, (char *)NULL,
 				     &header_from, 0, error);
 	    }
-	} else if (HDREQ("date", q->text)) {
+	} else if (HDREQ(date_fn, q->text)) {
 	    found_date = TRUE;
 	} else if (HDREQ(sender_fn, q->text)) {
 	    if (!sender_is_trusted) {
@@ -498,7 +501,11 @@ build_date_field()
 	(s = expand_string(date_field, (struct addr *)NULL,
 			   (char *)NULL, (char *)NULL)))
     {
-	return COPY_STRING(s);
+	if (use_resent) {
+	    return xprintf("Resent-%s", s);
+	} else {
+	    return COPY_STRING(s);
+	}
     }
     return "";
 }
