@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_95:log.c,v 1.20 1997/04/12 01:47:06 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_96:log.c,v 1.21 1997/05/23 17:50:58 woods Exp"
  */
 
 /*
@@ -383,6 +383,15 @@ panic(exitcode, fmt, va_alist)
     static int panic_count = 0;		/* panic not yet called once */
     va_list ap;
 
+    if (errfile) {
+#ifdef ANSI_C
+	va_start(ap, fmt);
+#else
+	va_start(ap);
+#endif
+	write_log_va(LOG_TTY, fmt, ap);
+	va_end(ap);
+    }
     /*
      * if panic has been called before, but the panic log is not open
      * write to the console.
@@ -431,14 +440,6 @@ panic(exitcode, fmt, va_alist)
 #endif
 	write_log_va(LOG_PANIC, fmt, ap);
 	va_end(ap);
-#ifdef ANSI_C
-	va_start(ap, fmt);
-#else
-	va_start(ap);
-#endif
-	if (errfile)
-	    write_log_va(LOG_TTY, fmt, ap);
-	va_end(ap);
     }
 
     if (daemon_pid != 0 && daemon_pid == getpid()) {
@@ -491,7 +492,8 @@ write_log(who, fmt, va_alist)
 #endif
     if (errfile && ((who & LOG_TTY) ||
 		   ((who & (LOG_MLOG|LOG_PANIC)) &&
-		    error_processing == TERMINAL &&
+		    (error_processing == TERMINAL ||
+		     error_processing == ERROR_DEFAULT) && /* XXX ??? */
 		    fmt[0] != 'X'))) {
 #ifdef ANSI_C
 	va_start(ap, fmt);
