@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_95:lookup.c,v 1.33 1997/05/05 15:33:39 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_96:lookup.c,v 1.34 1997/05/12 16:38:05 woods Exp"
  */
 
 /*
@@ -664,7 +664,7 @@ lsearch_lookup(db, key, value, error)
     char **value;
     char **error;
 {
-    int len = strlen(key);		/* length of comparison */
+    int len;				/* length of comparison */
     register char *entry;		/* entry from read_entry() */
     register char *p;			/* temp */
 
@@ -673,10 +673,15 @@ lsearch_lookup(db, key, value, error)
     /* always start from the beginning of the file */
     (void) fseek(db->f, 0L, 0);
 
+#ifndef USE_LSEARCH_REGEXCMP
+    len = strlen(key);			/* only once if that's all we need */
+#endif
+
     while ((entry = read_entry(db->f))) {
 	int rc;
 
 #ifdef USE_LSEARCH_REGEXCMP
+	len = strlen(key);
 	if (*entry == '\"')
 	    rc = lsearch_regexcmp(entry, key, &len);
 	else
