@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_95:main.c,v 1.40 1997/05/05 15:39:24 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_96:main.c,v 1.42 1997/05/23 17:55:20 woods Exp"
  */
 
 /*
@@ -177,28 +177,33 @@ main(argc, argv)
     argv++;
     sender_program = program;
 
+    /* initialize per-message state information */
+    initialize_state();
+
 #ifdef	UNIX_SCO
     /* if we don't have a login id, assume one */
     if (getluid() == -1)
-	setluid(0);
+	(void) setluid(0);
 #endif	/* UNIX_SCO */
 
 #ifdef	UNIX_AIX3
     /* if we don't have a login id, assume one */
     if (getuidx(ID_LOGIN) == -1)
-	setuidx(ID_LOGIN, 0);
+	(void) setuidx(ID_LOGIN, 0);
 #endif	/* UNIX_AIX3 */
 
 #ifdef	HAVE_SETGROUPS
     /* clear out all extra groups.  We don't want to have to deal with them */
     {
-	gid_t dummy[1];
-	dummy[0] = 0;
-# if (__FreeBSD__ <= 2)
-	(void) setgroups(1, dummy);
-# else
-	(void) setgroups(0, dummy);
-# endif
+	gid_t dummy_grouplist[1];
+	dummy_grouplist[0] = 0;
+
+	if (setgroups(0, (gid_t *) NULL) != 0) {
+	    if (setgroups(1, dummy_grouplist) != 0) {
+		write_log(LOG_TTY | ((getuid() == 0) ? LOG_PANIC : 0),
+			  "setgroups() failed: %s", strerror(errno));
+	    }
+	}
     }
 #endif	/* HAVE_SETGROUPS */
 
@@ -294,9 +299,6 @@ main(argc, argv)
 	prog_type = PROG_EXECMAIL;
     }
 
-    /* initialize per-message state information */
-    initialize_state();
-
     /* set state information which depends on program type */
     if (prog_type == PROG_NEWALIASES) {
 	operation_mode = REBUILD_ALIASES;
@@ -462,7 +464,8 @@ main(argc, argv)
 	    break;
 
 	default:
-	    panic(EX_OSFILE, "%s", error);
+	    write_log(LOG_TTY|LOG_PANIC, "%s", error);
+	    exit(EX_OSFILE);
 	    /*NOTREACHED*/
 	}
     }
@@ -482,7 +485,9 @@ main(argc, argv)
 	 * stat our binary so we can see if it has been touched later
 	 */
 	if (stat(smail, &statbuf) < 0) {
-	    panic(EX_SOFTWARE, "main: bad stat of smail binary %s", smail);
+	    write_log(LOG_TTY|LOG_PANIC, "main: bad stat() of smail binary %s", smail);
+	    exit(EX_SOFTWARE);
+	    /* NOTREACHED */
 	} else {
 	    add_config_stat(smail, &statbuf);
 	}
@@ -499,7 +504,9 @@ main(argc, argv)
 	    (error = read_qualify_file()) ||
 	    (error = read_retry_file()))
 	{
-	    panic(EX_OSFILE, "%s", error);
+	    write_log(LOG_TTY|LOG_PANIC, "%s", error);
+	    exit(EX_OSFILE);
+	    /* NOTREACHED */
 	}
 	break;
 
