@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_95:modes.c,v 1.62 1997/04/16 02:04:10 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_96:modes.c,v 1.64 1997/06/01 18:21:44 woods Exp"
  */
 
 /*
@@ -127,6 +127,14 @@
 # define SOMAXCONN	5
 #endif
 
+#ifdef HAVE_LIBWRAP
+
+# include <tcpd.h>  
+
+int smtp_session_denied = 0;		/* used in smtprecv.c */
+
+#endif /* HAVE_LIBWRAP */
+
 /* variables exported from this file */
 int daemon_pid = 0;
 
@@ -1197,23 +1205,31 @@ do_daemon_accept(ls, fd, from)
     if (pid == 0) {
 	FILE *in;			/* input channel */
 	FILE *out;			/* output channel */
+#ifdef HAVE_LIBWRAP
+	struct request_info req;
+#endif
 
 	/* in child process */
 	(void) close(ls);
-	in = fdopen(fd, "r");	/* setup the channels */
-	out = fdopen(fd2, "w");
+	/* setup the channels */
+	in = fdopen(fd, "r");		/* XXX error check this? */
+	out = fdopen(fd2, "w");		/* XXX error check this? */
 
 	/* no longer ignore kids */
 #ifdef SIGCHLD
 	(void) signal(SIGCHLD, SIG_DFL);
 #endif
 
+#ifdef HAVE_LIBWRAP
+	request_init(&req, RQ_DAEMON, program, RQ_FILE, fd, NULL);
+	fromhost(&req);
+	smtp_session_denied = !hosts_access(&req);
+#endif
 	/*
 	 * if the number of outstanding child processes exceeds
 	 * smtp_accept_queue, then turn on queue_only, so that
 	 * mail will not be delivered immediately.
 	 */
-
 	if (smtp_accept_queue > 0 && smtp_accept_count >= smtp_accept_queue) {
 	    DEBUG1(DBG_MAIN_MID, "many connections, use queue_only #%d...\n",
 		   smtp_accept_count + 1);
@@ -2079,7 +2095,11 @@ process_spool_file(spfn)
 
 	/* in child process, open the message and attempt delivery */
 	if (open_spool(spfn, TRUE, &error) == FAIL) {
-	    if (errno == 0) {
+	    if (errno == EAGAIN) {
+		DEBUG4(DBG_MAIN_LO, "open_spool: %s/%s: %s: %s.\n",
+		       spool_dir, input_spool_fn, error, strerror(errno));
+		exit(EX_TEMPFAIL);
+	    } else if (errno == 0) {
 		write_log(LOG_SYS, "open_spool: %s/%s: %s",
 			  spool_dir, input_spool_fn, error);
 	    } else {
