@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_95:smailconf.c,v 1.33 1997/04/16 02:08:27 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_96:smailconf.c,v 1.35 1997/05/23 17:57:35 woods Exp"
  */
 
 /*
@@ -140,6 +140,8 @@ static struct attr_table conf_attributes[] = {
     { "smtp_receive_message_timeout", t_interval, NULL,
       (tup *)&smtp_receive_message_timeout, 0 },
     { "smtp_remote_allow", t_string, NULL, (tup *)&smtp_remote_allow, 0 },
+    { "smtp_hello_verify", t_boolean, NULL, (tup *)&smtp_hello_verify, 0 },
+    { "smtp_sender_verify_mx_only", t_boolean, NULL, (tup *)&smtp_sender_verify_mx_only, 0 },
     { "spool_dirs", t_string, NULL, (tup *)&spool_dirs, 0 },
     { "spool_dir", t_string, NULL, (tup *)&spool_dir, 0 },
     { "spool_grade", t_char, NULL, (tup *)&spool_grade, 0 },
@@ -148,9 +150,9 @@ static struct attr_table conf_attributes[] = {
       (tup *)&switch_percent_and_bang, 0 },
     { "rfc1413_query_timeout", t_interval, NULL, (tup *)&rfc1413_query_timeout, 0 },
     { "transport_file", t_string, NULL, (tup *)&transport_file, 0 },
-    { "trusted", t_string, NULL, (tup *)&trusted, 0 },
     { "trusted_users", t_string, NULL, (tup *)&trusted, 0 },
     { "trusted_user", t_string, NULL, (tup *)&trusted, 0 },
+    { "trusted", t_string, NULL, (tup *)&trusted, 0 },
     { "trusted_groups", t_string, NULL, (tup *)&trusted_groups, 0 },
     { "trusted_group", t_string, NULL, (tup *)&trusted_groups, 0 },
     { "uucp_name", t_string, NULL, (tup *)&uucp_name, 0 },
