@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_95:smtprecv.c,v 1.35 1997/05/07 20:59:16 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_96:smtprecv.c,v 1.43 1997/06/01 18:33:38 woods Exp"
  */
 
 /*
@@ -28,6 +28,11 @@
 #include "hash.h"
 #include "alloc.h"
 #include "iobpeek.h"
+#ifdef HAVE_BIND
+# include "lookup.h"
+# include "bindlib.h"
+# include "route.h"
+#endif
 #ifndef DEPEND
 # include "extern.h"
 # include "debug.h"
@@ -40,14 +45,21 @@
 #include <ident.h>
 #endif /* HAVE_RFC1413 */
 
+#ifdef HAVE_LIBWRAP
+extern int smtp_session_denied;		/* from modes.c */
+#endif
+
 /* library functions */
 extern long time();
 
 /* Declare the ident variables - even if HAVE_RFC1413 is not defined, so that configs
  * can be kept consistant
  */
-char * ident_sender = NULL;		/* The calculated identity of the sender */
-char * ident_method = NULL;		/* Method used to get identity */
+char *ident_sender = NULL;		/* The calculated identity of the sender */
+char *ident_method = NULL;		/* Method used to get identity */
+
+/* classic IP guess at the local network (used by match_ip())*/
+char *smtp_local_net = NULL;
 
 /* types local to this file */
 enum e_smtp_commands {
@@ -72,7 +84,12 @@ enum e_smtp_commands {
 static void reset_state(void);
 static enum e_smtp_commands read_smtp_command(FILE *, FILE *);
 static void expand_addr(char *, FILE *);
+static int verify_sender(char *, char *, FILE *);
 static int verify_addr(char *, FILE *, int);
+static char *verify_host(char *, struct sockaddr_in *, char **);
+# if defined(HAVE_BIND) && defined(NEED_HSTRERROR)
+static char *hstrerror(int);
+# endif
 static void smtp_input_signals(void);
 static void smtp_processing_signals(void);
 static void set_term_signal(int);
@@ -85,7 +102,12 @@ static long compute_max_message_size_from_df_spool (void);
 static void reset_state();
 static enum e_smtp_commands read_smtp_command();
 static void expand_addr();
+static int verify_sender();
 static int verify_addr();
+static char *verify_host();
+# if defined(HAVE_BIND) && defined(NEED_HSTRERROR)
+static char *hstrerror();
+# endif
 static void smtp_input_signals();
 static void smtp_processing_signals();
 static void set_term_signal();
@@ -174,8 +196,8 @@ receive_smtp(in, out)
     int ehlo_p = 0;
     long accepted_msg_size = max_message_size;
 #ifdef HAVE_BSD_NETWORKING
-    struct sockaddr_in from;
-    int from_len = sizeof(from);
+    struct sockaddr_in from_sa;
+    int from_sa_len = sizeof(from_sa);
 #endif /* HAVE_BSD_NETWORKING */
 
     /* initialize state */
@@ -230,11 +252,49 @@ receive_smtp(in, out)
     }
 
 #ifdef HAVE_BSD_NETWORKING
-    if ((getpeername(fileno(in), (struct sockaddr *) &from, &from_len) == 0) &&
-	(from.sin_family == AF_INET)) {
-	/* make inet address available in ASCII representation */
-	sender_host_addr = COPY_STRING(inet_ntoa(from.sin_addr));
+    if (out &&
+	(getpeername(fileno(in), (struct sockaddr *) &from_sa, &from_sa_len) == 0) &&
+	(from_sa.sin_family == AF_INET)) {
+	struct sockaddr_in my_sa;
+	int my_sa_len = sizeof(my_sa);
+
+	/* INET_NTOA_USE_STRUCT not needed here, I think */
+
+	/* XXX if this ever got called twice for the same process it would leak */
+	sender_host_addr = COPY_STRING(inet_ntoa(from_sa.sin_addr));
+
+	if (getsockname(fileno(in), (struct sockaddr *) &my_sa, &my_sa_len) == 0 &&
+	    (my_sa.sin_family == AF_INET)) {
+	    char *p;
+
+	    /* XXX if this ever got called twice for the same process it would leak */
+	    smtp_local_net = COPY_STRING(inet_ntoa(inet_makeaddr(inet_netof(my_sa.sin_addr), 0)));
+
+	    /* now work backwards and replace ".0[.]" with ".*[.]" for match_ip() */
+	    for (p = smtp_local_net + strlen(smtp_local_net); p > smtp_local_net+2; p--) {
+		if ((*p == '.' || *p == '\0') && *(p-1) == '0' && *(p-2) == '.') {
+		    *(p-1) = '*';
+		    p -= 2;
+		} else if (*p && *p != '.' && *p != '0') {
+		    break;		/* don't s~204.0.102.0~204.*.102.*~! */
+		}
+	    }
+	    if ((p = index(smtp_local_net,'*'))) {
+		*(p+1) = '\0';		/* trunc. after first wildcard */
+	    }
+	} else {
+	    DEBUG1(DBG_DRIVER_LO, "getsockname(): %s.\n", strerror(errno));
+	    smtp_local_net = NULL;
+	}
+    } else {
+	DEBUG1(DBG_DRIVER_LO, "getpeername(): %s.\n", out ? strerror(errno) : "<no response chan>");
+	bzero((char *) &from_sa, sizeof(from_sa));
+	sender_host_addr = NULL;
+	smtp_local_net = NULL;
     }
+    DEBUG2(DBG_DRIVER_MID, "sender addr is %s, local net might be %s.\n",
+	   sender_host_addr ? sender_host_addr : "<unset>",
+	   smtp_local_net ? smtp_local_net : "<unset>");
 #endif /* HAVE_BSD_NETWORKING */
 
     /* Set the protocol used - this used to be in the HELO code, but that gives
@@ -254,24 +314,18 @@ receive_smtp(in, out)
      * available from an RFC1413 ident call, but I don't know what we would
      * do with the info, so don't bother to obtain it!
      */
-    if (rfc1413_query_timeout > 0) {	/* switch off RFC1413 by setting timeout <= 0 */
-	static char ident_str[32];
-	char * ident_ptr;
-
-	if (ident_ptr = ident_id(fileno(in), (int) rfc1413_query_timeout)) {
-	    strncpy(ident_str, ident_ptr, sizeof(ident_str));
-	    ident_str[sizeof(ident_str) - 1] = '\0';
-	    ident_sender = ident_str;
-	    free(ident_ptr);
+    if (out && rfc1413_query_timeout > 0) { /* switch off RFC1413 by setting timeout <= 0 */
+	if (ident_sender = ident_id(fileno(in), (int) rfc1413_query_timeout)) {
 	    ident_method = "rfc1413";
 	} else {
-	    ident_sender = NULL;
 	    ident_method = NULL;
 	}
     }
 #endif /* HAVE_RFC1413 */
 
     while (! term_signal || out == NULL) {
+	char *orig_data = NULL;
+
 	if (out) {
 	    alarm(smtp_receive_command_timeout);
 	}
@@ -281,30 +335,74 @@ receive_smtp(in, out)
 	    sender_proto = (out? "esmtp": "ebsmtp");
 	    /* FALLTHRU */
 	case HELO_CMD:
-	    strip_rfc822_comments(data);
-	    if (out && data[0] == '\0') {
-		fprintf(out, "501 %s requires domain name as operand\r\n",
-			ehlo_p ? "EHLO" : "HELO");
+#ifdef HAVE_LIBWRAP
+	    if (out && smtp_session_denied) {
+		fprintf(out, "550 You are not permitted to send mail\r\n");
 		fflush(out);
+		write_log(LOG_SYS, "remote %s: refusing SMTP connection from %s%s%s%s%s.",
+			  ehlo_p ? "EHLO" : "HELO",
+			  ident_sender ? ident_sender : "", ident_sender ? "@" : "",
+			  sender_host_addr ? "[" : "",
+			  sender_host_addr ? sender_host_addr : "",
+			  sender_host_addr ? "]" : "");
 		break;
 	    }
-	    if (sender_host == NULL && data[0] != '\0') {
+#endif
+	    orig_data = COPY_STRING(data);
+	    strip_rfc822_comments(data); /* XXX not required by RFC 821 */
+	    if (out) {
+		if (!data[0]) {
+		    fprintf(out, "501 %s requires host name as operand\r\n",
+			    ehlo_p ? "EHLO" : "HELO");
+		    fflush(out);
+		    break;
+		}
+	    }
+	    if (sender_host) {
+		xfree(sender_host);
+		sender_host = NULL;
+	    }
+	    if (out && smtp_hello_verify) {
+#ifdef HAVE_BSD_NETWORKING
+		char *error = NULL;
+
+		if (! (sender_host = verify_host(data, &from_sa, &error))) {
+		    fprintf(out, "501 %s requires a vaild host name as operand%s%s\r\n",
+			    ehlo_p ? "EHLO" : "HELO",
+			    error ? ": " : "", error ? error : "");
+		    fflush(out);
+		    write_log(LOG_SYS, "remote %s: invalid operand: %s%s%s%s%s%s%s%s%s.",
+			      ehlo_p ? "EHLO" : "HELO", orig_data,
+			      sender_host_addr ? ": remote address is " : "",
+			      ident_sender ? ident_sender : "",
+			      ident_sender ? "@" : "",
+			      sender_host_addr ? "[" : "",
+			      sender_host_addr ? sender_host_addr : "",
+			      sender_host_addr ? "]" : "",
+			      error ? ": " : "", error ? error : "");
+		    break;
+		}
+#endif /* HAVE_BSD_NETWORKING */
+	    } else if (!sender_host && data[0] != '\0') {
 		sender_host = COPY_STRING(data);
 	    }
-	    if (!ehlo_p) {
-		if (out) {
-		    fprintf(out, "250 %s Hello %s\r\n", primary_name, data);
-		    fflush(out);
-		}
-	    } else {
-		if (out) {
-		    fprintf(out, "250-%s Hello %s, here's what we support:\r\n",
-			    primary_name, data);
+	    if (out) {
+		fprintf(out, "250%s%s Hello %s (%s%s%s%s%s%s)%s\r\n",
+			ehlo_p ? "-" : " ",
+			primary_name, data,
+			ident_sender ? ident_sender : "",
+			ident_sender ? "@" : "",
+			sender_host,
+			sender_host_addr ? " [" : "",
+			sender_host_addr ? sender_host_addr : "",
+			sender_host_addr ? "]" : "",
+			ehlo_p ? ", here's what we support:" : ".");
+		if (ehlo_p) {
 #ifndef NO_SMTP_EXPN
 		    fprintf(out, "250-EXPN\r\n");
 #endif
 #ifdef HAVE_DF_SPOOL
-		    accepted_msg_size = compute_max_message_size_from_df_spool ();
+		    accepted_msg_size = compute_max_message_size_from_df_spool();
 		    if (accepted_msg_size == -1 
 			|| (max_message_size && max_message_size < accepted_msg_size))
 			accepted_msg_size = max_message_size;
@@ -321,16 +419,35 @@ receive_smtp(in, out)
 		    fprintf(out, "250-PIPELINING\r\n");
 #endif
 		    fprintf(out, "250 HELP\r\n");
-		    fflush(out);
 		}
+		(void) fflush(out);
 	    }
 	    reset_state();
 	    break;
 
 	case MAIL_CMD:
+#ifdef HAVE_LIBWRAP
+	    if (out && smtp_session_denied) {
+		fprintf(out, "550 You are not permitted to send mail\r\n");
+		fflush(out);
+		write_log(LOG_SYS, "remote MAIL FROM: '%s' refusing SMTP connection from %s%s%s%s%s%s%s",
+			  data, ident_sender ? ident_sender : "", ident_sender ? "@" : "",
+			  sender_host ? sender_host : "<HELO/EHLO not given>",
+			  sender_host_addr ? " [" : "",
+			  sender_host_addr ? sender_host_addr : "",
+			  sender_host_addr ? "]" : "");
+		break;
+	    }
+#endif
+	    orig_data = COPY_STRING(data);
 	    strip_rfc822_comments(data);
 	    if (out && data[0] == '\0') {
-		fprintf(out, "501 MAIL FROM requires address as operand\r\n");
+		fprintf(out, "501 MAIL FROM requires return address as operand.\r\n");
+		fflush(out);
+		break;
+	    }
+	    if (out && !sender_host) {
+		fprintf(out, "503 MAIL FROM must be preceded by HELO/EHLO command.\r\n");
 		fflush(out);
 		break;
 	    }
@@ -342,12 +459,15 @@ receive_smtp(in, out)
 		break;
 	    }
 	    sender = preparse_address_1(data, &error, &rest);
-  	    if (out) {
-		if (!sender) {
-  		    fprintf(out, "501 <%s> ... %s\r\n", data, error);
-		    break;
-  		}
-  	    }
+	    if (out && !sender) {
+		fprintf(out, "501 <%s> ... %s\r\n", data, error);
+		fflush(out);
+		break;
+ 	    }
+	    if (out && !verify_sender(sender, data, out)) {
+		sender = NULL;
+		break;
+	    }
 	    if (sender && sender[0] == '\0') {
 		/* special error sender form <> given */
 		sender = COPY_STRING("<>");
@@ -356,7 +476,7 @@ receive_smtp(in, out)
 		/* special smail-internal <+> was given */
 		sender = COPY_STRING("<+>");
 	    }
-	    {
+	    {				/* XXX this should be a sub-function too! */
 		char * format = 0;
 		int format_length = 0;
 #ifdef HAVE_ESMTP_8BITMIME
@@ -458,26 +578,52 @@ receive_smtp(in, out)
 		if (out) {
 		    if (format && *format != '\0') {
 			*(format+format_length) = '\0';
-			fprintf(out, "250 <%s> ... Sender Okay, using format %s\r\n",
+			fprintf(out, "250 %s ... Sender Okay, using format %s\r\n",
 				sender, format);
 		    } else {
-			fprintf(out, "250 <%s> ... Sender Okay\r\n",
+			fprintf(out, "250 %s ... Sender Okay\r\n",
 				sender);
 		    }
 		}
 	    }
+	    if (out) {
+		fflush(out);
+	    }
 	    break;
 	  fail_mail_cmd:
 	    if (sender) {
 		xfree(sender);
 		sender = NULL;
 	    }
+	    if (out) {
+		fflush(out);
+	    }
 	    break;
 
 	case RCPT_CMD:
+#ifdef HAVE_LIBWRAP
+	    if (out && smtp_session_denied) {
+		fprintf(out, "550 You are not permitted to send mail\r\n");
+		fflush(out);
+		write_log(LOG_SYS, "remote MAIL TO: '%s' refusing SMTP connection from %s%s%s%s%s%s%s",
+			  data, ident_sender ? ident_sender : "", ident_sender ? "@" : "",
+			  sender_host ? sender_host : "<HELO/EHLO not given>",
+			  sender_host_addr ? " [" : "",
+			  sender_host_addr ? sender_host_addr : "",
+			  sender_host_addr ? "]" : "");
+		break;
+	    }
+#endif
+	    orig_data = COPY_STRING(data);
 	    strip_rfc822_comments(data);
 	    if (out && data[0] == '\0') {
-		fprintf(out, "501 RCPT TO requires address as operand\r\n");
+		fprintf(out, "501 RCPT TO requires address as operand.\r\n");
+		fflush(out);
+		break;
+	    }
+	    if (!sender) {
+		fprintf(out, "501 RCPT TO must be preceded by MAIL FROM command.\r\n");
+		fflush(out);
 		break;
 	    }
 	    cur = alloc_addr();
@@ -503,7 +649,7 @@ receive_smtp(in, out)
 	case DATA_CMD:
 	    if (sender == NULL) {
 		if (out) {
-		    fprintf(out, "503 Need MAIL command\r\n");
+		    fprintf(out, "503 Need MAIL FROM command first.\r\n");
 		    fflush(out);
 		} else {
 		    /* sink the message for the sake of further batched cmds */
@@ -516,7 +662,7 @@ receive_smtp(in, out)
 	    }
 	    if (recipients == NULL) {
 		if (out) {
-		    fprintf(out, "503 Need RCPT (recpient)\r\n");
+		    fprintf(out, "503 DATA command must be preceded by RCPT TO command.\r\n");
 		    fflush(out);
 		} else {
 		    /* sink the message for the sake of further batched cmds */
@@ -612,6 +758,7 @@ receive_smtp(in, out)
 		fprintf(out, SMTP_RPLY_252);
 #else
 		if (smtp_info) {
+		    orig_data = COPY_STRING(data);
 		    strip_rfc822_comments(data);
 		    verify_addr(data, out, 0);
 		} else {
@@ -623,11 +770,25 @@ receive_smtp(in, out)
 	    break;
 
 	case EXPN_CMD:
+#ifdef HAVE_LIBWRAP
+	    if (out && smtp_session_denied) {
+		fprintf(out, "550 You not permitted to view lists.\r\n");
+		fflush(out);
+		write_log(LOG_SYS, "remote EXPN: '%s' refusing SMTP connection from %s%s%s%s%s%s%s",
+			  data, ident_sender ? ident_sender : "", ident_sender ? "@" : "",
+			  sender_host ? sender_host : "<HELO/EHLO not given>",
+			  sender_host_addr ? " [" : "",
+			  sender_host_addr ? sender_host_addr : "",
+			  sender_host_addr ? "]" : "");
+		break;
+	    }
+#endif
 	    if (out) {
 #ifdef NO_SMTP_EXPN			/* XXX should be a variable ala smtp_debug */
 		fprintf(out, "502 Command not implemented\r\n");
 #else
 		if (smtp_info) {
+		    orig_data = COPY_STRING(data);
 		    strip_rfc822_comments(data);
 		    expand_addr(data, out);
 		} else {
@@ -640,8 +801,7 @@ receive_smtp(in, out)
 
 	case QUIT_CMD:
 	    if (out) {
-		fprintf(out, "221 %s closing connection\r\n",
-			       primary_name);
+		fprintf(out, "221 %s closing connection\r\n", primary_name);
 		fflush(out);
 	    }
 	    reset_state();
@@ -654,6 +814,7 @@ receive_smtp(in, out)
 	    reset_state();
 	    if (out) {
 		fprintf(out, "250 Reset state\r\n");
+		fflush(out);
 	    }
 	    break;
 
@@ -665,9 +826,23 @@ receive_smtp(in, out)
 	    break;
 
 	case DEBUG_CMD:
+#ifdef HAVE_LIBWRAP
+	    if (out && smtp_session_denied) {
+		fprintf(out, "550 You are not permitted to use command.\r\n");
+		fflush(out);
+		write_log(LOG_SYS, "remote DEBUG: '%s' refusing SMTP connection from %s%s%s%s%s%s%s",
+			  data, ident_sender ? ident_sender : "", ident_sender ? "@" : "",
+			  sender_host ? sender_host : "<HELO/EHLO not given>",
+			  sender_host_addr ? " [" : "",
+			  sender_host_addr ? sender_host_addr : "",
+			  sender_host_addr ? "]" : "");
+		break;
+	    }
+#endif
 	    if (out) {
 #ifndef NODEBUG
 		if (smtp_debug) {
+		    orig_data = COPY_STRING(data);
 		    strip_rfc822_comments(data);
 		    if (data[0]) {
 			char *errbuf = NULL;
@@ -725,6 +900,10 @@ receive_smtp(in, out)
 	    }
 	    break;
 	}
+	if (orig_data) {
+	    xfree(orig_data);
+	    orig_data = NULL;
+	}
     }
 
     /*
@@ -758,6 +937,9 @@ reset_state()
     }
     recipients = NULL;
 
+    /* FYI, do not zap sender_host_addr or any ident stuff, or any other TCP
+     * connection related stuff
+     */
     if (sender) {
 	xfree(sender);
 	sender = NULL;
@@ -849,6 +1031,14 @@ expand_addr(in_addr, out)
     struct addr *fail = NULL;		/* list of undeliverable addrs */
     char *error;			/* hold error message */
 
+# ifndef NO_LOG_SMTP_EXPN
+    write_log(LOG_SYS, "remote EXPN: '%s' by %s%s%s%s%s%s.", in_addr,
+	      ident_sender ? ident_sender : "", ident_sender ? "@" : "",
+	      sender_host ? sender_host : "<HELO/EHLO not given>",
+	      sender_host_addr ? " [" : "",
+	      sender_host_addr ? sender_host_addr : "",
+	      sender_host_addr ? "]" : "");
+# endif
     addr->in_addr = in_addr;		/* setup the input addr structure */
     /* build the mungeable addr string */
     addr->work_addr = preparse_address(in_addr, &error);
@@ -889,6 +1079,145 @@ expand_addr(in_addr, out)
 }
 #endif	/* !NO_SMTP_EXPN */
 
+/*
+ * verify_sender - verify envelope sender address
+ */
+static int
+verify_sender(pp_sender, raw_sender, out)
+    char *pp_sender;			/* from preparse_address_1() */
+    char *raw_sender;			/* full data from user */
+    FILE *out;				/* file for response codes */
+{
+    struct addr *vaddr;
+    int form;
+
+    vaddr = alloc_addr();
+    vaddr->in_addr = COPY_STRING(raw_sender);
+    vaddr->work_addr = COPY_STRING(pp_sender);
+    form = parse_address(vaddr->work_addr, &vaddr->target,
+			 &vaddr->remainder, &vaddr->parseflags);
+    if (*vaddr->work_addr && (form == FAIL || form == PARSE_ERROR)) {
+	fprintf(out, "501 '%s' address parse error: %s.\r\n", vaddr->work_addr,
+		(vaddr->remainder) ? vaddr->remainder : "<unknown error>");
+	fflush(out);
+	xfree(vaddr->in_addr);
+	xfree(vaddr->work_addr);
+	xfree(vaddr);
+	return 0;
+    }
+    if (*vaddr->work_addr && !EQ(vaddr->work_addr, "+") && form == LOCAL) {
+	fprintf(out, "501 '%s' sender address cannot be a local address.\r\n", vaddr->work_addr);
+	fflush(out);
+	write_log(LOG_SYS, "remote MAIL FROM: '%s' is a local address; by %s%s%s%s%s%s.",
+		  vaddr->in_addr,
+		  ident_sender ? ident_sender : "", ident_sender ? "@" : "",
+		  sender_host,
+		  sender_host_addr ? " [" : "",
+		  sender_host_addr ? sender_host_addr : "",
+		  sender_host_addr ? "]" : "");
+	xfree(vaddr->in_addr);
+	xfree(vaddr->work_addr);
+	xfree(vaddr);
+	return 0;
+    }
+#ifdef HAVE_BIND
+    /*
+     * XXX What about checking if it is a ROUTE addr?
+     * XXX What about other forms (eg. UUCP_ROUTE, BERKNET, DECNET)?
+     */
+    if (*vaddr->work_addr && !EQ(vaddr->work_addr, "+")) {
+	int mxresult;
+	long bindflgs = BIND_DOMAIN_REQUIRED | BIND_DEFER_NO_CONN | BIND_LOCAL_MX_OKAY;
+	struct rt_info rt_info;
+	static struct error errst = { 0L, NULL };
+	struct error *binderr = &errst;
+	static struct bindlib_private bindpriv =  BIND_TEMPLATE_ATTRIBUTES;
+
+	rt_info.next_host = NULL;
+	rt_info.route = NULL;
+	rt_info.transport = NULL;
+	rt_info.tphint_list = NULL;
+
+	if (smtp_sender_verify_mx_only) {
+	    bindflgs |= BIND_MX_ONLY;	/* i.e. don't look for A RRs (avoid term servers) */
+	}
+
+	mxresult = bind_addr(vaddr->target, bindflgs, &bindpriv,
+			     "verify_sender()", &rt_info, &binderr);
+
+	DEBUG2(DBG_DRIVER_MID, "verify_sender(): bind_addr(): %d, %s.\n", mxresult,
+	       (binderr->message && *binderr->message) ? binderr->message : "no error");
+
+	if (rt_info.next_host) {
+	    xfree(rt_info.next_host);	/* clean up alloc'ed storage we don't need */
+	}
+	if (rt_info.route) {
+	    xfree(rt_info.route);	/* clean up alloc'ed storage we don't need */
+	}
+
+	switch (mxresult) {
+	case DB_SUCCEED:
+	    return 1;
+
+	case DB_AGAIN:			/* DNS lookup must be deferred */
+	case FILE_AGAIN:		/* lost contact with server */
+	case FILE_NOMATCH:		/* There is no server available */
+	    fprintf(out, "450 defer '%s', sender address cannot be verifyied at this time: %s.\r\n",
+		    vaddr->in_addr,
+		    (binderr->message && *binderr->message) ? binderr->message : "Unknown error");
+	    fflush(out);
+	    xfree(vaddr->in_addr);
+	    xfree(vaddr->work_addr);
+	    xfree(vaddr);
+	    return 0;
+
+	case DB_NOMATCH:		/* no such domain */
+	    fprintf(out, "550 '%s' sender address is not a valid domain%s.\r\n",
+		    vaddr->in_addr, smtp_sender_verify_mx_only ? " (no MX record)" : "");
+	    fflush(out);
+	    write_log(LOG_SYS, "remote MAIL FROM: '%s' not a valid domain%s; by %s%s%s%s%s%s.",
+		      vaddr->in_addr, smtp_sender_verify_mx_only ? " (no MX record)" : "",
+		      ident_sender ? ident_sender : "", ident_sender ? "@" : "",
+		      sender_host,
+		      sender_host_addr ? " [" : "",
+		      sender_host_addr ? sender_host_addr : "",
+		      sender_host_addr ? "]" : "");
+	    xfree(vaddr->in_addr);
+	    xfree(vaddr->work_addr);
+	    xfree(vaddr);
+	    return 0;
+
+	case DB_FAIL:			/* bad DNS request? */
+	case FILE_FAIL:			/* major DNS error! */
+	    fprintf(out, "550 '%s' failure trying to verify address: %s.\r\n", vaddr->in_addr,
+		    (binderr->message && *binderr->message) ? binderr->message : "Unknown error");
+	    fflush(out);
+	    write_log(LOG_SYS, "remote MAIL FROM: '%s' DNS failed: %s; by %s%s%s%s%s%s.",
+		      vaddr->in_addr,
+		      (binderr->message && *binderr->message) ? binderr->message : "Unknown error",
+		      ident_sender ? ident_sender : "", ident_sender ? "@" : "",
+		      sender_host,
+		      sender_host_addr ? " [" : "",
+		      sender_host_addr ? sender_host_addr : "",
+		      sender_host_addr ? "]" : "");
+	    xfree(vaddr->in_addr);
+	    xfree(vaddr->work_addr);
+	    xfree(vaddr);
+	    return 0;
+
+	default:
+	    panic(EX_SOFTWARE, "verify_sender(): bind_addr() gave impossible result %d.", mxresult);
+	    /* NOTREACHED */
+	}
+    }
+#endif
+    xfree(vaddr->in_addr);
+    xfree(vaddr->work_addr);
+    xfree(vaddr);
+
+    return 1;
+}
+
 /*
  * verify_addr - verify an address
  *
@@ -911,6 +1240,7 @@ verify_addr(in_addr, out, rcpt_p)
     addr->work_addr = preparse_address(in_addr, &error);
     if (addr->work_addr == NULL) {
 	fprintf(out, "501 '%s' malformed address; %s.\r\n", in_addr, error);
+	xfree(addr);
 	return 0;
     }
 
@@ -926,20 +1256,24 @@ verify_addr(in_addr, out, rcpt_p)
     verify_addr_list(addr, &okay, &defer, &fail);
 
     if (okay) {
-	fprintf(out, "250 <%s@%s> Recipient Okay.\r\n",
+	fprintf(out, "250 '%s' <%s@%s> Recipient Okay.\r\n", in_addr,
 		okay->work_addr, (okay->target) ? okay->target : "(nodomain)");
+	xfree(addr);
  	return 1;
     } else if (defer) {
- 	fprintf(out, "450 <%s@%s> cannot verify at this time: (ERR_%03ld) %s.\r\n",
- 		defer->work_addr, (defer->target)? defer->target : "(nodomain)",
+	/* XXX hope that the other guy eventually gives up */
+ 	fprintf(out, "450 defer '%s' <%s@%s> cannot verify at this time: (ERR_%03ld) %s.\r\n",
+		in_addr, defer->work_addr,
+		(defer->target) ? defer->target : "(nodomain)",
 		defer->error->info & ERR_MASK, defer->error->message);
+	xfree(addr);
  	return 1;
     } else if (fail) {
 	int error_code;			/* reply code for negative result */
 
 	switch (fail->error->info & ERR_MASK) {
 	case ERR_104:
-	    error_code = 551;		/* mailbox not local */
+	    error_code = 551;		/* XXX 550? mailbox not local */
 	    break;
 	case ERR_111:
 	    error_code = 501;		/* syntax error in parameter */
@@ -955,32 +1289,182 @@ verify_addr(in_addr, out, rcpt_p)
 	    error_code = 553;		/* use this to flag un-xlated errors */
 	    break;
 	}
- 	fprintf(out, "%d <%s@%s> not matched: %s.\r\n", error_code,
+ 	fprintf(out, "%d '%s' <%s@%s> not matched: %s.\r\n", error_code, in_addr,
 		fail->work_addr, (fail->target) ? fail->target : "(nodomain)",
 		fail->error->message);
 	if (rcpt_p) {
-	    write_log(LOG_SYS, "remote %s: '%s' <%s@%s> recipient not matched: %s",
+	    write_log(LOG_SYS, "remote %s%s%s%s%s%s: '%s' <%s@%s> recipient not matched: %s",
+		      ident_sender ? ident_sender : "", ident_sender ? "@" : "",
+		      sender_host,
+		      sender_host_addr ? "[" : "",
 		      sender_host_addr ? sender_host_addr : "<no-sender-addr>",
-		      in_addr, fail->work_addr,
-		      (fail->target) ? fail->target : "(nodomain)",
+		      sender_host_addr ? "]" : "",
+		      in_addr, fail->work_addr, (fail->target) ? fail->target : "(nodomain)",
 		      fail->error->message);
 	}
- 	return 0;
-    } else {
-  	/* hmmm, it should have been in one of the lists */
- 	fprintf(out, "550 <%s@%s> does not match anything!\r\n",
-		addr->work_addr, (addr->target) ? addr->target : "(nodomain)");
-	if (rcpt_p) {
-	    /* XXX do we really need to log these? */
-	    write_log(LOG_SYS, "remote %s: '%s' <%s@%s> recipient not matched.",
-		      sender_host_addr ? sender_host_addr : "(no-sender-addr)",
-		      in_addr, addr->work_addr,
-		      (addr->target) ? addr->target : "(nodomain)");
-	}
+	xfree(addr);
  	return 0;
     }
+    /* hmmm, it should have been in one of the lists from verify_addr_list()! */
+    fprintf(out, "550 '%s' <%s@%s> does not match anything!\r\n", in_addr,
+	    addr->work_addr, (addr->target) ? addr->target : "(nodomain)");
+    if (rcpt_p) {
+	/* XXX do we really need to log these? */
+	write_log(LOG_SYS, "remote %s%s%s%s%s%s: '%s' <%s@%s> recipient not matched",
+		  ident_sender ? ident_sender : "", ident_sender ? "@" : "",
+		  sender_host,
+		  sender_host_addr ? "[" : "",
+		  sender_host_addr ? sender_host_addr : "<no-sender-addr>",
+		  sender_host_addr ? "]" : "",
+		  in_addr, addr->work_addr,
+		  (addr->target) ? addr->target : "(nodomain)");
+    }
+    xfree(addr);
+    return 0;
 }
 
+
+#ifdef HAVE_BSD_NETWORKING
+/*
+ * verify_host - verify the sender hostname in the DNS
+ *
+ * If the hostname is an address literal then we first match it with the
+ * sender_host_addr and return NULL if it doesn't match.  If it does match we
+ * try looking up the PTR RR for it, and if we find one we return a copy of
+ * that name, otherwise we fail with NULL.
+ *
+ * [[Address Literals *should* be followed by helpful information....]]
+ *
+ * For all other hostnames we look up the A RR for the name supplied, the PTR
+ * RR for the resulting A RR, verify that they match, then verify that the
+ * sender_host_address also matches.
+ */
+static char *
+verify_host(hostnm, saddrp, errorp)
+    char *hostnm;
+    struct sockaddr_in *saddrp;
+    char **errorp;
+{
+    struct hostent *hp;
+    int found, i;
+
+    extern int h_errno;			/* import h_errno; many systems don't define it in <netdb.h> */
+
+    if (*hostnm == '[') {
+# if defined(INET_ADDR_USE_STRUCT)
+	struct in_addr s_inet;		/* internet address */
+# endif
+	unsigned long inet;		/* internet address */
+	char *p = index(hostnm, ']');
+	struct str hn;			/* returned host name */
+
+	if (!p) {
+	    DEBUG1(DBG_DRIVER_LO, "verify_host(%s): Invalid host address.\n", hostnm);
+	    *errorp = "Invalid host address";
+	    return NULL;
+	}
+	*p = '\0';
+# ifdef INET_ADDR_USE_STRUCT
+	s_inet = inet_addr(&hostnm[1]);
+	inet = s_inet.s_addr;
+# else
+	inet = inet_addr(&hostnm[1]);
+# endif
+	*p = ']';
+	if (inet == (unsigned long)(-1)) {
+	    DEBUG1(DBG_DRIVER_LO, "verify_host(%s) bad host address form.\n", hostnm);
+	    *errorp = "Invalid host address form";
+	    return NULL;
+	}
+	if (inet != saddrp->sin_addr.s_addr) {
+	    DEBUG3(DBG_DRIVER_LO, "verify_host(%s): [0x%lx] != [0x%lx].\n", hostnm, inet, saddrp->sin_addr.s_addr);
+	    *errorp = "Host address does not match socket address";
+	    return NULL;
+	}
+	STR_INIT(&hn);
+	*p = '\0';
+	STR_CAT(&hn, hostnm);		/* always return the literal given */
+	STR_CAT(&hn, "]");		/* the bracket we clobbered just before */
+	*p = ']';
+	if (!(hp = gethostbyaddr((char *) &inet, INADDRSZ, AF_INET))) {
+	    *errorp = (char *) hstrerror(h_errno);
+	    DEBUG2(DBG_DRIVER_LO, "verify_host(%s) gethostbyaddr() failed: %s.\n", hostnm, *errorp);
+	} else {
+	    STR_CAT(&hn, "(");
+	    STR_CAT(&hn, hp->h_name);	/* return the actual name as found by the PTR RR in a comment */
+	    STR_CAT(&hn, ")");
+	}
+	if (*++p) {
+	    STR_CAT(&hn, "(");
+	    STR_CAT(&hn, p);		/* also patch on another comment containing the remainder */
+	    STR_CAT(&hn, ")");
+	}
+	
+	return STR(&hn);
+    } /* else not a literal */
+
+    if (! (hp = gethostbyname(hostnm))) {
+	*errorp = (char *) hstrerror(h_errno);
+	DEBUG2(DBG_DRIVER_LO, "verify_host(%s) gethostbyname() failed: %s.\n", hostnm, *errorp);
+	return NULL;
+    }
+    found = 0;
+    for (i = 0; hp->h_addr_list[i]; i++) {
+	if (bcmp(hp->h_addr_list[i], (char *) &saddrp->sin_addr,
+		 sizeof(saddrp->sin_addr)) == 0) {
+	    found = 1;
+	    break;			/* name is good, keep it */
+	}
+    }
+    if (!found) {
+	*errorp = "Host name does not match socket address";
+	DEBUG2(DBG_DRIVER_LO, "verify_host(%s) gethostbyname() doesn't find %s.\n", hostnm, inet_ntoa(saddrp->sin_addr));
+	return NULL;
+    }
+
+    /* XXX We should probably check that there is a matching PTR too, but both
+     * the DRUMS SMTPupd draft, and RFC 1123 (which SMTPupd will replace), say
+     * that the smtp-SERVER must not reject connections just because some
+     * arbitrary verification of the HELO/EHLO parameter doesn't wash.  I
+     * disagree vehemently, but the above is probably enough to foil the truely
+     * stupid. */
+
+    return COPY_STRING(hostnm);
+}
+#endif
+
+#if defined(HAVE_BSD_NETWORKING) && defined(NEED_HSTRERROR)
+static char *
+hstrerror(h_err)
+    int h_err;
+{
+    /* XXX Should use h_errlist if it's available */
+    switch (h_errno) {
+    case 0:
+	return "Resolver Error 0 (no error)";
+
+    case NO_DATA:
+	return "No DNS data";
+
+    case HOST_NOT_FOUND:
+	return "Unknown host";
+
+    case TRY_AGAIN:
+	return "Hostname lookup failure";
+
+    case NO_RECOVERY:
+	return "Unknown server error";
+
+    case NO_ADDRESS:
+	return "No address associated with name";
+
+    default:
+	return "Unknown resolver error";
+    }
+    /* NOTREACHED */
+}
+#endif
+
 
 /*
  * smtp_input_signals - setup signals for reading in message with smtp
@@ -1050,7 +1534,7 @@ smtp_receive_timeout_sig(sig)
 {
     fprintf(out_file, "421 %s SMTP command timeout, closing channel\r\n",
 	    primary_name);
-    write_log(LOG_SYS, "SMTP connection timeout%s%s.",
+    write_log(LOG_SYS, "SMTP connection timeout%s%s%s%s%s.",
 	      sender_host? " while talking with ": "",
 	      sender_host? sender_host: "",
 	      sender_host_addr? " [": "",
