@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/transports:RELEASE-3_2_0_95:smtplib.c,v 1.24 1997/04/19 22:22:38 woods Exp"
+#ident	"@(#)smail/src/transports:RELEASE-3_2_0_96:smtplib.c,v 1.26 1997/06/01 17:35:13 woods Exp"
  */
 
 /*
@@ -107,7 +107,7 @@ static int write_command_nowait(struct smtp *, char *, int);
 static int flush_command_stream(struct smtp *, char **);
 static int wait_read_response(struct smtp *, unsigned, char **);
 static int read_response_internal(struct smtp *, unsigned, char **);
-static void catch_timeout(void);
+static void catch_timeout(int);
 static struct error * no_remote(struct transport *, char *);
 static struct error * try_again(struct transport *, char *);
 static struct error * fatal_error(struct transport *, char *);
@@ -249,20 +249,19 @@ smtp_startup(smtpb, error_p, try_ehlo)
 		    on_greet_line = 0;
 		    goto skip_rest_of_line;
 		}
-		for (keywordlength = 0;
-		     *(cp+keywordlength) != 0
-		     && !isspace (*(cp+keywordlength));
-		     ++keywordlength)
-		    ;
+		keywordlength = 0;
+		while (*(cp+keywordlength) != 0 && !isspace(*(cp+keywordlength))) {
+		     ++keywordlength;
+		}
 		if (strncmpic(cp, "SIZE", keywordlength) == 0) {
 		    unsigned long max_size = 0;
 
 		    cp += keywordlength;
 		    while (*cp == ' ' || *cp == '\t')
 			++cp;
-		    if (!isdigit(*cp) && *cp != '\n')
+		    if (*cp && !isdigit(*cp) && *cp != '\n')
 			goto malformed_ehlo_reply;
-		    while (isdigit(*cp)) {
+		    while (*cp && isdigit(*cp)) {
 			max_size *= 10;
 			max_size += *cp - '0';
 			++cp;
@@ -280,6 +279,9 @@ smtp_startup(smtpb, error_p, try_ehlo)
 		} else if (strncmpic(cp, "XQUE", keywordlength) == 0) {
 		    smtpb->smtp_flags |= ESMTP_queue;
 		} else {
+		    char sc = *(cp+keywordlength);
+		    DEBUG1(DBG_DRIVER_MID, "destination offers unsupported ESMTP option %s.\n", cp);
+		    *(cp+keywordlength) = sc;
 		}
 	      skip_rest_of_line:
 		while (*cp != 0 && *cp != '\n')
@@ -287,19 +289,13 @@ smtp_startup(smtpb, error_p, try_ehlo)
 		if (*cp == '\n')
 		    ++cp;
 	    }
-# ifndef NODEBUG
-	    /* not worth defining DEBUG6()! */
-	    if (debug > DBG_REMOTE_MID && errfile) {
-		dprintf(errfile, "destination supports ESMTP%s%s%s%s%s%s",
-		      smtpb->smtp_flags & ESMTP_8bitmime ? " 8BITMIME" : "",
-		      smtpb->smtp_flags & ESMTP_size ? " SIZE" : "",
-		      smtpb->smtp_flags & ESMTP_pipelining ? " PIPELINING" : "",
-		      smtpb->smtp_flags & ESMTP_verbose ? " XVRB" : "",
-		      smtpb->smtp_flags & ESMTP_one ? " XONE" : "",
-		      smtpb->smtp_flags & ESMTP_queue ? " XQUE" : "");
-		fflush(errfile);
-	    }
-# endif
+	    DEBUG6(DBG_DRIVER_MID, "destination supports ESMTP%s%s%s%s%s%s.\n",
+		   smtpb->smtp_flags & ESMTP_8bitmime ? " 8BITMIME" : "",
+		   smtpb->smtp_flags & ESMTP_size ? " SIZE" : "",
+		   smtpb->smtp_flags & ESMTP_pipelining ? " PIPELINING" : "",
+		   smtpb->smtp_flags & ESMTP_verbose ? " XVRB" : "",
+		   smtpb->smtp_flags & ESMTP_one ? " XONE" : "",
+		   smtpb->smtp_flags & ESMTP_queue ? " XQUE" : "");
 	    if (smtpb->smtp_flags & ESMTP_verbose
 		&& debug >= DBG_DRIVER_MID) {
 		smtp_out.i = 0;
@@ -317,6 +313,7 @@ smtp_startup(smtpb, error_p, try_ehlo)
 	    /* This seems to be a reasonable way
 	       to handle malformed EHLO replies: */
 # ifndef NO_LOG_EHLO
+	    /* XXX this is really ugly as it leaves the newlines in place... */
 	    write_log(LOG_SYS, "destination supports esmtp, but is buggy (%s)",
 		      reply_text);
 # endif /* not NO_LOG_EHLO */
@@ -912,8 +909,10 @@ read_response_internal(smtpb, timeout, reply_text)
 }
 
 /* catch_timeout - longjmp after an alarm */
+/* ARGSUSED */
 static void
-catch_timeout()
+catch_timeout(signo)
+	int	signo;
 {
     JUMP_LONGJMP(timeout_buf, 1);
 }
