@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/transports:RELEASE-3_2_0_95:tcpsmtp.c,v 1.29 1997/04/12 01:48:01 woods Exp"
+#ident	"@(#)smail/src/transports:RELEASE-3_2_0_96:tcpsmtp.c,v 1.30 1997/05/23 17:46:33 woods Exp"
  */
 
 /*
@@ -673,6 +673,10 @@ ip_address(remote_host,ipaddr,family,error)
     short *family;
     char **error;
 {
+#ifdef INET_ADDR_USE_STRUCT
+	struct in_addr s_inet;		/* internet address */
+#endif
+
     if (remote_host[0] == '[') {
         /* INET addr literal address */
         char *p = index(remote_host, ']');
@@ -683,7 +687,12 @@ ip_address(remote_host,ipaddr,family,error)
             return -1;
         } else {
             *p = '\0';
+#ifdef INET_ADDR_USE_STRUCT
+            s_inet_number = inet_addr(remote_host + 1);
+	    inet_number = s_inet_number.s_addr;
+#else
             inet_number = inet_addr(remote_host + 1);
+#endif
             *p = ']';
         }
         ipaddr->s_addr = inet_number;
