@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_95:verify.c,v 1.8 1997/04/21 18:04:28 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_96:verify.c,v 1.10 1997/06/03 01:41:07 woods Exp"
  */
 
 /*
@@ -221,6 +221,8 @@ verify_addr_list(in, okay, defer, fail)
  * Takes a list of IP:IP:IP and compares against an IP string.  Allows
  * "*" star type wildcards at the end of IPs.
  *
+ * WARNING: cannot diddle with pattern -- may be read-only string.
+ *
  * Returns 1 if matched, else 0.
  */
 static int
@@ -231,14 +233,14 @@ match_ip(ip, pattern)
     char	*pat_p, *end_p, wild = 0;
     int		len;
     
-    /* run through the pattern */
-    for (pat_p = pattern;; pat_p = end_p + 1) {
-	
+    /* run through the pattern list */
+    for (pat_p = pattern; /* NOTEST */ ; pat_p = end_p + 1, wild = 0) {
+
 	/* skip any spaces at front */
-	for (; *pat_p == ' ' || *pat_p == '\t' || *pat_p == '\n'; pat_p++);
+	for (; *pat_p && isspace(*pat_p); pat_p++);
 	
-	/* find the end of the pattern */
-	for (end_p = pat_p; *end_p != ':' && *end_p != '\0'; end_p++);
+	/* find the end of the next pattern */
+	for (end_p = pat_p; *end_p && *end_p != ':' && !isspace(*pat_p); end_p++);
 	
 	/* skip empty patterns */
 	len = end_p - pat_p;
@@ -247,22 +249,35 @@ match_ip(ip, pattern)
 	    if (end_p > pat_p && *(end_p - 1) == '*') {
 		len--;
 		wild = 1;
-	    }
-	    
-	    /* have match if pat was wild or we checked the whole IP */
-	    if (strncmp(ip, pat_p, len) == 0 && (wild || ip[len] == '\0')) {
-		DEBUG3(DBG_QUEUE_HI,
-		       "ip '%s' verified by sub-pattern '%s', len %d\n",
-		       ip, pat_p, len);
-		return 1;
+	    } else if (smtp_local_net && strncmpic("localnet", pat_p, len) == 0) {
+		/* our pattern will be the string in smtp_local_net */
+		len = strlen(smtp_local_net);
+		if (smtp_local_net[len - 1] == '*') {
+		    len--;
+		    wild = 1;
+		}
+		/* have match if pat was wild or we checked the whole IP */
+		if (strncmp(ip, smtp_local_net, len) == 0 && (wild || ip[len] == '\0')) {
+		    DEBUG3(DBG_ADDR_HI,
+			   "ip '%s' verified by localnet '%s', up to len %d\n",
+			   ip, smtp_local_net, len);
+		    return 1;
+		}
+	    } else {
+		/* have match if pat was wild or we checked the whole IP */
+		if (strncmp(ip, pat_p, len) == 0 && (wild || ip[len] == '\0')) {
+		    DEBUG3(DBG_ADDR_HI,
+			   "ip '%s' verified by sub-pattern of '%s', up to len %d\n",
+			   ip, pat_p, len);
+		    return 1;
+		}
 	    }
 	}
-	
 	if (*end_p == '\0')
-	  break;
+	    break;
     }
     
-    DEBUG2(DBG_QUEUE_HI, "ip '%s' not verified by pattern '%s'\n",
+    DEBUG2(DBG_ADDR_HI, "ip '%s' not verified by pattern '%s'\n",
 	   ip, pattern);
     
     return 0;
