@@ -1,4 +1,4 @@
-/* @(#) mkdbm.c,v 1.9 1996/02/16 14:57:44 woods Exp */
+/* @(#) mkdbm.c,v 1.10 1997/05/05 16:37:00 woods Exp */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -24,11 +24,12 @@
 #include "smail.h"
 #include "dys.h"
 #include "exitcodes.h"
-#ifdef	UNIX_BSD
-# include <sys/file.h>
-#endif
-#ifdef	UNIX_SYS5
+#if defined(UNIX_SYS5) || defined(POSIX_OS) || defined(USE_FCNTL)
 # include <fcntl.h>
+#else
+# if defined(UNIX_BSD)
+#  include <sys/file.h>
+# endif
 #endif
 #include "dbm_compat.h"
 #ifndef DEPEND
